@@ -9,8 +9,8 @@ import sys
 from contextlib import contextmanager, suppress
 from functools import partial
 from typing import (
-    Any, Callable, Dict, Generator, Iterable, List, Match, NamedTuple,
-    Optional, Sequence, Set, Tuple, Type, Union
+    Any, Callable, Dict, Generator, Iterable, List, NamedTuple, Optional,
+    Sequence, Set, Tuple, Type, Union
 )
 
 from . import fast_data_types as defines
@@ -19,48 +19,23 @@ from .conf.utils import (
     BadLine, init_config, key_func, load_config as _load_config, merge_dicts,
     parse_config_base, python_string, to_bool, to_cmdline
 )
-from .config_data import all_options, parse_mods, type_convert
+from .config_data import InvalidMods, all_options, parse_shortcut, type_convert
 from .constants import cache_dir, defconf, is_macos
-from .key_names import get_key_name_lookup, key_name_aliases
+from .fonts import FontFeature
 from .options_stub import Options as OptionsStub
+from .types import SingleKey
 from .typing import TypedDict
-from .utils import log_error
+from .utils import expandvars, log_error
 
-KeySpec = Tuple[int, bool, int]
-KeyMap = Dict[KeySpec, 'KeyAction']
-KeySequence = Tuple[KeySpec, ...]
+KeyMap = Dict[SingleKey, 'KeyAction']
+KeySequence = Tuple[SingleKey, ...]
 SubSequenceMap = Dict[KeySequence, 'KeyAction']
-SequenceMap = Dict[KeySpec, SubSequenceMap]
-
-
-class InvalidMods(ValueError):
-    pass
-
-
-def parse_shortcut(sc: str) -> Tuple[int, bool, Optional[int]]:
-    parts = sc.split('+')
-    mods = 0
-    if len(parts) > 1:
-        mods = parse_mods(parts[:-1], sc) or 0
-        if not mods:
-            raise InvalidMods('Invalid shortcut')
-    q = parts[-1].upper()
-    key: Optional[int] = getattr(defines, 'GLFW_KEY_' + key_name_aliases.get(q, q), None)
-    is_native = False
-    if key is None:
-        q = parts[-1]
-        if q.startswith('0x'):
-            with suppress(Exception):
-                key = int(q, 16)
-        else:
-            key = get_key_name_lookup()(q, False)
-        is_native = key is not None
-    return mods, is_native, key
+SequenceMap = Dict[SingleKey, SubSequenceMap]
 
 
 class KeyAction(NamedTuple):
     func: str
-    args: Sequence[str]
+    args: Sequence[str] = ()
 
 
 func_with_args, args_funcs = key_func()
@@ -140,6 +115,20 @@ def float_parse(func: str, rest: str) -> FuncArgsType:
     return func, (float(rest),)
 
 
+@func_with_args('signal_child')
+def signal_child_parse(func: str, rest: str) -> FuncArgsType:
+    import signal
+    signals = []
+    for q in rest.split():
+        try:
+            signum = getattr(signal, q.upper())
+        except AttributeError:
+            log_error(f'Unknown signal: {rest} ignoring')
+        else:
+            signals.append(signum)
+    return func, tuple(signals)
+
+
 @func_with_args('change_font_size')
 def parse_change_font_size(func: str, rest: str) -> Tuple[str, Tuple[bool, Optional[str], float]]:
     vals = rest.strip().split(maxsplit=1)
@@ -240,6 +229,15 @@ def set_colors(func: str, rest: str) -> FuncArgsType:
     return func, r
 
 
+@func_with_args('remote_control')
+def remote_control(func: str, rest: str) -> FuncArgsType:
+    import shlex
+    r = shlex.split(rest)
+    if len(r) < 1:
+        log_error('Too few arguments to remote_control function')
+    return func, r
+
+
 @func_with_args('nth_window')
 def nth_window(func: str, rest: str) -> FuncArgsType:
     try:
@@ -300,11 +298,12 @@ def parse_marker_spec(ftype: str, parts: Sequence[str]) -> Tuple[str, Union[str,
 
 @func_with_args('toggle_marker')
 def toggle_marker(func: str, rest: str) -> FuncArgsType:
+    import shlex
     parts = rest.split(maxsplit=1)
     if len(parts) != 2:
-        raise ValueError('{} if not a valid marker specification'.format(rest))
+        raise ValueError('{} is not a valid marker specification'.format(rest))
     ftype, spec = parts
-    parts = spec.split()
+    parts = shlex.split(spec)
     return func, list(parse_marker_spec(ftype, parts))
 
 
@@ -347,15 +346,22 @@ sequence_sep = '>'
 
 class KeyDefinition:
 
-    def __init__(self, is_sequence: bool, action: KeyAction, mods: int, is_native: bool, key: int, rest: Tuple[KeySpec, ...] = ()):
+    def __init__(self, is_sequence: bool, action: KeyAction, mods: int, is_native: bool, key: int, rest: Tuple[SingleKey, ...] = ()):
         self.is_sequence = is_sequence
         self.action = action
-        self.trigger = mods, is_native, key
+        self.trigger = SingleKey(mods, is_native, key)
         self.rest = rest
 
     def resolve(self, kitty_mod: int) -> None:
-        self.trigger = defines.resolve_key_mods(kitty_mod, self.trigger[0]), self.trigger[1], self.trigger[2]
-        self.rest = tuple((defines.resolve_key_mods(kitty_mod, mods), is_native, key) for mods, is_native, key in self.rest)
+
+        def r(k: SingleKey) -> SingleKey:
+            mods = defines.resolve_key_mods(kitty_mod, k.mods)
+            key = k.key
+            is_native = k.is_native
+            return SingleKey(mods, is_native, key)
+
+        self.trigger = r(self.trigger)
+        self.rest = tuple(map(r, self.rest))
 
     def resolve_kitten_aliases(self, aliases: Dict[str, Sequence[str]]) -> None:
         if not self.action.args:
@@ -370,7 +376,7 @@ class KeyDefinition:
                 if len(expanded) > 1:
                     rest = expanded[1] + ' ' + rest
         if changed:
-            self.action = self.action._replace(args=[kitten + (' ' + rest).rstrip()])
+            self.action = self.action._replace(args=[kitten, rest.rstrip()])
 
 
 def parse_key(val: str, key_definitions: List[KeyDefinition]) -> None:
@@ -383,28 +389,28 @@ def parse_key(val: str, key_definitions: List[KeyDefinition]) -> None:
         return
     is_sequence = sequence_sep in sc
     if is_sequence:
-        trigger: Optional[Tuple[int, bool, int]] = None
-        restl: List[Tuple[int, bool, int]] = []
+        trigger: Optional[SingleKey] = None
+        restl: List[SingleKey] = []
         for part in sc.split(sequence_sep):
             try:
                 mods, is_native, key = parse_shortcut(part)
             except InvalidMods:
                 return
-            if key is None:
+            if key == 0:
                 if mods is not None:
                     log_error('Shortcut: {} has unknown key, ignoring'.format(sc))
                 return
             if trigger is None:
-                trigger = mods, is_native, key
+                trigger = SingleKey(mods, is_native, key)
             else:
-                restl.append((mods, is_native, key))
+                restl.append(SingleKey(mods, is_native, key))
         rest = tuple(restl)
     else:
         try:
             mods, is_native, key = parse_shortcut(sc)
         except InvalidMods:
             return
-        if key is None:
+        if key == 0:
             if mods is not None:
                 log_error('Shortcut: {} has unknown key, ignoring'.format(sc))
             return
@@ -501,14 +507,6 @@ def handle_symbol_map(key: str, val: str, ans: Dict[str, Any]) -> None:
     ans['symbol_map'].update(parse_symbol_map(val))
 
 
-class FontFeature(str):
-
-    def __new__(cls, name: str, parsed: bytes) -> 'FontFeature':
-        ans: FontFeature = str.__new__(cls, name)  # type: ignore
-        ans.parsed = parsed  # type: ignore
-        return ans
-
-
 @special_handler
 def handle_font_features(key: str, val: str, ans: Dict[str, Any]) -> None:
     if val != 'none':
@@ -576,20 +574,6 @@ def handle_deprecated_macos_show_window_title_in_menubar_alias(key: str, val: st
     ans['macos_show_window_title_in'] = macos_show_window_title_in
 
 
-def expandvars(val: str, env: Dict[str, str]) -> str:
-
-    def sub(m: Match) -> str:
-        key = m.group(1)
-        result = env.get(key)
-        if result is None:
-            result = os.environ.get(key)
-        if result is None:
-            result = m.group()
-        return result
-
-    return re.sub(r'\$\{(\S+?)\}', sub, val)
-
-
 @special_handler
 def handle_env(key: str, val: str, ans: Dict[str, Any]) -> None:
     key, val = val.partition('=')[::2]
@@ -709,40 +693,6 @@ def cached_values_for(name: str) -> Generator[Dict, None, None]:
             err))
 
 
-def initial_window_size_func(opts: OptionsStub, cached_values: Dict) -> Callable[[int, int, float, float, float, float], Tuple[int, int]]:
-
-    if 'window-size' in cached_values and opts.remember_window_size:
-        ws = cached_values['window-size']
-        try:
-            w, h = map(int, ws)
-
-            def initial_window_size(*a: Any) -> Tuple[int, int]:
-                return w, h
-            return initial_window_size
-        except Exception:
-            log_error('Invalid cached window size, ignoring')
-
-    w, w_unit = opts.initial_window_width
-    h, h_unit = opts.initial_window_height
-
-    def get_window_size(cell_width: int, cell_height: int, dpi_x: float, dpi_y: float, xscale: float, yscale: float) -> Tuple[int, int]:
-        if not is_macos:
-            # scaling is not needed on Wayland, but is needed on macOS. Not
-            # sure about X11.
-            xscale = yscale = 1
-        if w_unit == 'cells':
-            width = cell_width * w / xscale + (dpi_x / 72) * (opts.window_margin_width + opts.window_padding_width) + 1
-        else:
-            width = w
-        if h_unit == 'cells':
-            height = cell_height * h / yscale + (dpi_y / 72) * (opts.window_margin_width + opts.window_padding_width) + 1
-        else:
-            height = h
-        return int(width), int(height)
-
-    return get_window_size
-
-
 def commented_out_default_config() -> str:
     ans = []
     for line in as_conf_file(all_options.values()):
