@@ -14,6 +14,7 @@
 #include <poll.h>
 #include <pthread.h>
 #include "glfw-wrapper.h"
+#include "banned.h"
 // Required minimum OpenGL version
 #define OPENGL_REQUIRED_VERSION_MAJOR 3
 #define OPENGL_REQUIRED_VERSION_MINOR 3
@@ -37,10 +38,15 @@
 #define zero_at_ptr_count(p, count) memset((p), 0, (count) * sizeof((p)[0]))
 void log_error(const char *fmt, ...) __attribute__ ((format (printf, 1, 2)));
 #define fatal(...) { log_error(__VA_ARGS__); exit(EXIT_FAILURE); }
+static inline void cleanup_free(void *p) { free(*(void**)p); }
+#define FREE_AFTER_FUNCTION __attribute__((cleanup(cleanup_free)))
 
 typedef unsigned long long id_type;
 typedef uint32_t char_type;
 typedef uint32_t color_type;
+typedef uint16_t hyperlink_id_type;
+typedef int key_type;
+#define HYPERLINK_MAX_NUMBER UINT16_MAX
 typedef uint16_t combining_type;
 typedef uint32_t pixel;
 typedef unsigned int index_type;
@@ -74,8 +80,6 @@ typedef enum { TILING, SCALED, MIRRORED } BackgroundImageLayout;
 #define ATTRS_MASK_FOR_SGR (ATTRS_MASK_WITHOUT_MARK | ATTRS_MASK_WITHOUT_WIDTH)
 #define MARK_MASK 3
 #define COL_MASK 0xFFFFFFFF
-#define UTF8_ACCEPT 0
-#define UTF8_REJECT 1
 #define DECORATION_FG_CODE 58
 #define CHAR_IS_BLANK(ch) ((ch) == 32 || (ch) == 0)
 #define CONTINUED_MASK 1
@@ -147,6 +151,8 @@ typedef enum { TILING, SCALED, MIRRORED } BackgroundImageLayout;
 #endif
 
 
+typedef enum UTF8State { UTF8_ACCEPT = 0, UTF8_REJECT = 1} UTF8State;
+
 typedef struct {
     uint32_t left, top, right, bottom;
 } Region;
@@ -160,6 +166,7 @@ typedef struct {
 typedef struct {
     char_type ch;
     combining_type cc_idx[2];
+    hyperlink_id_type hyperlink_id;
 } CPUCell;
 
 
@@ -190,13 +197,19 @@ typedef struct {
 } HistoryBufSegment;
 
 typedef struct {
-    index_type bufsize, maxsz;
-    Py_UCS4 *buffer;
-    index_type start, end;
-    index_type bufend;
+    void *ringbuf;
+    size_t maximum_size;
     bool rewrap_needed;
 } PagerHistoryBuf;
 
+typedef struct {int x;} *HYPERLINK_POOL_HANDLE;
+typedef struct {
+    Py_UCS4 *buf;
+    size_t len, capacity;
+    HYPERLINK_POOL_HANDLE hyperlink_pool;
+    hyperlink_id_type active_hyperlink_id;
+} ANSIBuf;
+
 typedef struct {
     PyObject_HEAD
 
@@ -228,14 +241,20 @@ typedef struct {
     color_type default_fg, default_bg, cursor_color, cursor_text_color, cursor_text_uses_bg, highlight_fg, highlight_bg;
 } DynamicColor;
 
+
+typedef struct {
+    DynamicColor dynamic_colors;
+    uint32_t color_table[256];
+} ColorStackEntry;
+
 typedef struct {
     PyObject_HEAD
 
     bool dirty;
     uint32_t color_table[256];
     uint32_t orig_color_table[256];
-    DynamicColor dynamic_color_stack[10];
-    size_t dynamic_color_stack_idx;
+    ColorStackEntry *color_stack;
+    unsigned int color_stack_idx, color_stack_sz;
     DynamicColor configured, overridden;
     color_type mark_foregrounds[MARK_MASK+1], mark_backgrounds[MARK_MASK+1];
 } ColorProfile;
@@ -253,17 +272,6 @@ typedef struct {FONTS_DATA_HEAD} *FONTS_DATA_HANDLE;
 
 #define clear_sprite_position(cell) (cell).sprite_x = 0; (cell).sprite_y = 0; (cell).sprite_z = 0;
 
-#define left_shift_line(line, at, num) { \
-    for(index_type __i__ = (at); __i__ < (line)->xnum - (num); __i__++) { \
-        COPY_CELL(line, __i__ + (num), line, __i__) \
-    } \
-    if ((((line)->gpu_cells[(at)].attrs) & WIDTH_MASK) != 1) { \
-        (line)->cpu_cells[(at)].ch = BLANK_CHAR; \
-        (line)->gpu_cells[(at)].attrs = BLANK_CHAR ? 1 : 0; \
-        clear_sprite_position((line)->gpu_cells[(at)]); \
-    }\
-}
-
 #define ensure_space_for(base, array, type, num, capacity, initial_cap, zero_mem) \
     if ((base)->capacity < num) { \
         size_t _newcap = MAX((size_t)initial_cap, MAX(2 * (base)->capacity, (size_t)num)); \
@@ -294,8 +302,8 @@ void cursor_reset(Cursor*);
 Cursor* cursor_copy(Cursor*);
 void cursor_copy_to(Cursor *src, Cursor *dest);
 void cursor_reset_display_attrs(Cursor*);
-void cursor_from_sgr(Cursor *self, unsigned int *params, unsigned int count);
-void apply_sgr_to_cells(GPUCell *first_cell, unsigned int cell_count, unsigned int *params, unsigned int count);
+void cursor_from_sgr(Cursor *self, int *params, unsigned int count);
+void apply_sgr_to_cells(GPUCell *first_cell, unsigned int cell_count, int *params, unsigned int count);
 const char* cell_as_sgr(const GPUCell *, const GPUCell *);
 const char* cursor_as_sgr(const Cursor *);
 
@@ -306,16 +314,15 @@ bool set_iutf8(int, bool);
 color_type colorprofile_to_color(ColorProfile *self, color_type entry, color_type defval);
 float cursor_text_as_bg(ColorProfile *self);
 void copy_color_table_to_buffer(ColorProfile *self, color_type *address, int offset, size_t stride);
-void colorprofile_push_dynamic_colors(ColorProfile*);
-void colorprofile_pop_dynamic_colors(ColorProfile*);
+bool colorprofile_push_colors(ColorProfile*, unsigned int);
+bool colorprofile_pop_colors(ColorProfile*, unsigned int);
+void colorprofile_report_stack(ColorProfile*, unsigned int*, unsigned int*);
 
 void set_mouse_cursor(MouseShape);
 void enter_event(void);
 void mouse_event(int, int, int);
 void focus_in_event(void);
 void scroll_event(double, double, int, int);
-void fake_scroll(int, bool);
-void set_special_key_combo(int glfw_key, int mods, bool is_native);
 void on_key_input(GLFWkeyevent *ev);
 void request_window_attention(id_type, bool);
 #ifndef __APPLE__
@@ -323,6 +330,6 @@ void play_canberra_sound(const char *which_sound, const char *event_id);
 #endif
 SPRITE_MAP_HANDLE alloc_sprite_map(unsigned int, unsigned int);
 SPRITE_MAP_HANDLE free_sprite_map(SPRITE_MAP_HANDLE);
+const char* get_hyperlink_for_id(const HYPERLINK_POOL_HANDLE, hyperlink_id_type id, bool only_url);
 
-static inline void safe_close(int fd) { while(close(fd) != 0 && errno == EINTR); }
 void log_event(const char *format, ...) __attribute__((format(printf, 1, 2)));
